import sys
import traceback

import multiply
import studentGrades
import utstudent
import utils
   
IS_MISSING = -1

def approxEqual(value, target, tolerance):
   if (abs(target) <= tolerance ): return abs(value) <= tolerance
   return abs(float(value) / target - 1) <= tolerance
   
def check(name, value, target, tolerance):
   if approxEqual(value, target, tolerance):
      return True
   else:
      print("\nWrong %s: your value %f, correct value %f"
               % (name, value, target))
      return False

def q1(testFileName):

   print("Running multiplication test: " + str(testFileName) + "...", end='')
   
   try:
      with open(testFileName, "r") as testFile:
         # Read test information
         try:
            fileLines = testFile.read().splitlines()
            pointsPossible = IS_MISSING
            for line in fileLines:
               # Ignore comments and blank lines
               if len(line.strip()) == 0 or line[0] == '#':
                  continue
                   
               # Set points possible
               if pointsPossible == IS_MISSING:
                  pointsPossible = int(line)
                  continue
                  
               # Read problem data
               problemData = line.split(",")
               factor1 = float(problemData[0])
               factor2 = float(problemData[1])
               product = float(problemData[2])
                  
         except:
            print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
            traceback.print_exc(file=sys.stdout)
            return 0, 0
            
         # Now run the actual test
         try:
            testProduct = multiply.calculateProduct(factor1, factor2)
            correct = check("product", testProduct, product, 0.0001)  
            if correct == False:               
               print("...fail")
               return 0, pointsPossible
         except utils.NotYetAttemptedException:
            print("...not yet attempted")
            return 0, pointsPossible
         except:
            print("\nException raised, attempting to continue:")
            traceback.print_exc(file=sys.stdout)                     
            print("\n...fail")
            return 0, pointsPossible
            
         print("...pass")
         return pointsPossible, pointsPossible

         
   except IOError:
      print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
      traceback.print_exc(file=sys.stdout) 
      return 0, 0
   

def q2(testFileName):

   print("Running data structures test: " + str(testFileName) + "...", end='')
   
   try:
      with open(testFileName, "r") as testFile:
         # Read test information
         try:
            fileLines = testFile.read().splitlines()
            pointsPossible = IS_MISSING
            studentRecords = list()
            for line in fileLines:
               # Ignore comments and blank lines
               if len(line.strip()) == 0 or line[0] == '#':
                  continue
                   
               # Set points possible
               if pointsPossible == IS_MISSING:
                  pointsPossible = int(line)
                  continue
                  
               # Read problem data; the last line is the correct answer
               problemData = line.split(",")
               if len(problemData) == 1: # Average
                  average = float(problemData[0])
               else:
                  studentRecords.append( (problemData[0], float(problemData[1])) )   
                                 
         except:
            print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
            traceback.print_exc(file=sys.stdout)
            return 0, 0
            
         # Now run the actual test
         try:
            testAverage = studentGrades.calculateAverage(studentRecords[:-1])
            (highName, highScore) = studentGrades.findHighScore(studentRecords[:-1])
            score = 0
            score += 1 if check("average", testAverage, average, 0.0001) == True else 0
            score += 1 if highName == studentRecords[-1][0] else 0
            score += 1 if check("high score", highScore, studentRecords[-1][1], 0) == True else 0
            
            if (score < pointsPossible):   
               if highName != studentRecords[-1][0]:
                  print()            
                  print("Top student is " + studentRecords[-1][0] + ", your code said it was " + highName)                  
               print("...fail")
               return score, pointsPossible
         except utils.NotYetAttemptedException:
            print("...not yet attempted")
            return 0, pointsPossible
         except:
            print("\nException raised, attempting to continue:")
            traceback.print_exc(file=sys.stdout)                     
            print("\n...fail")
            return 0, pointsPossible
            
         print("...pass")
         return score, pointsPossible

         
   except IOError:
      print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
      traceback.print_exc(file=sys.stdout) 
      return 0, 0
   

def q3(testFileName):

   print("Running classes test: " + str(testFileName) + "...", end='')
   
   try:
      with open(testFileName, "r") as testFile:
         # Read test information
         try:
            fileLines = testFile.read().splitlines()
            pointsPossible = IS_MISSING
            listOfStudents = list()
            for line in fileLines:
               # Ignore comments and blank lines
               if len(line.strip()) == 0 or line[0] == '#':
                  continue
                   
               # Set points possible
               if pointsPossible == IS_MISSING:
                  pointsPossible = int(line)
                  continue
                  
               # Read problem data; if there is only one  is the correct answer
               problemData = line.split(",")
               if len(problemData) == 1: # Average
                  correctTopEID = problemData[0]
               else:
                  listOfStudents.append( utstudent.UTStudent(problemData[0], problemData[1], float(problemData[2]), float(problemData[3])))
                                 
         except:
            print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
            traceback.print_exc(file=sys.stdout)
            return 0, 0
            
         # Now run the actual test
         try:
            topEID = utstudent.findHighestEID(listOfStudents)
            if topEID != correctTopEID:
               print()
               print("Top EID is " + correctTopEID + ", your code said it was " + topEID)
               print("...fail")
               return 0, pointsPossible
         except utils.NotYetAttemptedException:
            print("...not yet attempted")
            return 0, pointsPossible
         except:
            print("\nException raised, attempting to continue:")
            traceback.print_exc(file=sys.stdout)                     
            print("\n...fail")
            return 0, pointsPossible
            
         print("...pass")
         return pointsPossible, pointsPossible

         
   except IOError:
      print("\nError running test %s, attempting to continue with remaining tests.  Exception details: " % testFileName)
      traceback.print_exc(file=sys.stdout) 
      return 0, 0
   

